#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <signal.h>
#include "define.h"
#include "code.h"

void mul128(DATA_TYPE a, DATA_TYPE b, DATA_TYPE *rh, DATA_TYPE *rl){
  DATA_TYPE ah, al, bh, bl;
  DATA_TYPE x, y, z;
  int carry=0; /* has a value of 0, 1 and 2 */
    
  ah = a >> 32;
  al = a & 0xFFFFFFFF;
  
  bh = b >> 32;
  bl = b & 0xFFFFFFFF;

  x = al*bl;
  y = (al*bh << 32);
  z = x + y;
  if(z<x || z<y) carry ++;
  x = z;
  y = (ah*bl << 32);
  z = x + y;
  if(z<x || z<y) carry ++;
  *rl = z;
  
  *rh = ah*bh+ (al*bh >> 32) + (ah*bl >> 32) + carry;
}


DATA_TYPE zap(int byte_mask, DATA_TYPE ret){
  if(byte_mask & BIT0) ret &=  ZAP0;
  if(byte_mask & BIT1) ret &=  ZAP1;
  if(byte_mask & BIT2) ret &=  ZAP2;
  if(byte_mask & BIT3) ret &=  ZAP3;
  if(byte_mask & BIT4) ret &=  ZAP4;
  if(byte_mask & BIT5) ret &=  ZAP5;
  if(byte_mask & BIT6) ret &=  ZAP6;
  if(byte_mask & BIT7) ret &=  ZAP7;
  return ret;
}

/*****
* There are four instruction types;
* (1)PAL Code Type ; Code<31:26>            Function_code<25:0>            ;
* (2)Branch Type   ; Code<31:26> Ra<25:21>                  Diff<20:0>     ;
* (3)Memory Type   ; Code<31:26> Ra<25:21> Rb<20:16>        Diff<15:0>     ;
* (4)Operation Type; Code<31:26> Ra<25:21> Rb<20:16> Function<15:5> Rc<4:0>;
 *****/


DATA_TYPE ex_cmov(int code, unsigned int func11,
		  DATA_TYPE Rav, DATA_TYPE Rbv, DATA_TYPE Rmv){
  DATA_TYPE Rcv=0;

  switch(code){
  case 0x11:
    switch(func11 & 0x7F)
      {
      case 0x24: /* __CMOVEQ; */
	if(Rav==0) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x46: /* __CMOVGE; */
	if((long long)Rav>=0) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x66: /* __CMOVGT; */
	if((long long)Rav>0) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x16: /* __CMOVLBC; */
	if(!(Rav & 1)) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x14: /* __CMOVLBS; */
	if(Rav & 1) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x64: /*__CMOVLE; */
	if((long long)Rav<=0) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x44: /*__CMOVLT; */
	if((long long)Rav<0) Rcv = Rbv; else Rcv = Rmv;
	break;
      case 0x26: /* __CMOVNE; */
	if(Rav!=0) Rcv = Rbv; else Rcv = Rmv;
	break;
      default:
	cout <<  "Warning 0x11 CMOV Default\n";
	Rcv=0;
	break;
      }
    break;

  case 0x17:
    {
      double Fav=0.0, Fbv=0.0, Fcv=0.0, Fmv=0.0;
      memcpy(&Fav, &Rav, 8);
      memcpy(&Fbv, &Rbv, 8);
      memcpy(&Fmv, &Rmv, 8);
      
      switch(func11){
      case 0x02a:/*__FCMOVEQ; */
	if(Fav==0.0) Fcv=Fbv; else Fcv=Fmv;
	break;
      case 0x02b:/*__FCMOVNE; */
	if(Fav!=0.0) Fcv=Fbv; else Fcv=Fmv;
	break;
      default:
	printf("Warning 0x17 Floating CMOV func11 0x%x\n",func11);
	Fcv=0.0;
	break;
      }

      memcpy(&Rcv, &Fcv, 8);
      break;
    }
  }
  return Rcv;
}
  
DATA_TYPE ex_arith(int code, unsigned int func11,
		   DATA_TYPE Rav, DATA_TYPE Rbv){
  DATA_TYPE Rcv=0;
  DATA_TYPE tmp, tmp2, tmp3;
  unsigned int byte_mask, byte_loc;

  switch(code){
  case 0x10:
    switch(func11 & 0x7F)
      {
      case 0x00:
	Rcv = ((Rav + Rbv) & MASK32);
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x20: /* __ADDQ; */
	Rcv = Rav + Rbv;
	break;
      case 0x09: /* __SUBL; */
	Rcv = (Rav - Rbv) & MASK32;
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x29: /* __SUBQ; */
	Rcv = Rav - Rbv;
	break;
      case 0x2d: /* __CMPEQ; */
	Rcv = ((long long)Rav == (long long)Rbv);
	break;
      case 0x4d: /* __CMPLT; */
	Rcv = ((long long)Rav <  (long long)Rbv);
	break;
      case 0x6d: /* __CMPLE; */
	Rcv = ((long long)Rav <= (long long)Rbv);
	break;
      case 0x3d: /* __CMPULE; */
	Rcv = (Rav <= Rbv);
	break;
      case 0x1d: /* __CMPULT; */
	Rcv = (Rav < Rbv);
	break;
      case 0x02: /* __S4ADDL; */
	Rcv = ((Rav << 2) + Rbv) & MASK32;
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x12: /* __S8ADDL; */
	Rcv = ((Rav << 3) + Rbv) & MASK32;
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x22: /* __S4ADDQ; */
	Rcv = (Rav << 2) + Rbv;
	break;
      case 0x32: /* __S8ADDQ; */
	Rcv = (Rav << 3) + Rbv;
	break;
      case 0x0b: /* __S4SUBL; */
	Rcv = ((Rav << 2) - Rbv) & MASK32;
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x1b: /* __S8SUBL; */
	Rcv = ((Rav << 3) - Rbv) & MASK32;
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x2b: /* __S4SUBQ; */
	Rcv = (Rav << 2) - Rbv;
	break;
      case 0x3b: /* __S8SUBQ; */
	Rcv = (Rav << 3) - Rbv;
	break;
      case 0x0f: /*__CMPBGE; */
	Rcv=0;
	if((Rav & ~ZAP0) >= (Rbv & ~ZAP0)) Rcv |= BIT0;
	if((Rav & ~ZAP1) >= (Rbv & ~ZAP1)) Rcv |= BIT1;
	if((Rav & ~ZAP2) >= (Rbv & ~ZAP2)) Rcv |= BIT2;
	if((Rav & ~ZAP3) >= (Rbv & ~ZAP3)) Rcv |= BIT3;
	if((Rav & ~ZAP4) >= (Rbv & ~ZAP4)) Rcv |= BIT4;
	if((Rav & ~ZAP5) >= (Rbv & ~ZAP5)) Rcv |= BIT5;
	if((Rav & ~ZAP6) >= (Rbv & ~ZAP6)) Rcv |= BIT6;
	if((Rav & ~ZAP7) >= (Rbv & ~ZAP7)) Rcv |= BIT7;
	break;
	
      case 0x40: printf("Not implemented addl/v\n"); Rcv=0; break;
      case 0x60: printf("Not implemented addq/v\n"); Rcv=0; break;
      case 0x49: printf("Not implemented subl/v\n"); Rcv=0; break;
      case 0x69: printf("Not implemented subq/v\n"); Rcv=0; break;

      default:
	printf("Warning in 0x10 arithmetic.\n");
	Rcv=0;
      }
    break;
            
  case 0x11:
    switch(func11 & 0x7F)
      {
      case 0x00: /* __AND; */
	Rcv = Rav &  Rbv;
	break;
      case 0x20: /* __BIS; */
	Rcv = Rav |  Rbv;
	break;
      case 0x40: /* __XOR; */
	Rcv = Rav ^  Rbv;
	break;
      case 0x08: /* __BIC; */
	Rcv = Rav & ~Rbv;
	break;
      case 0x28: /* __ORNOT; */
	Rcv = Rav | ~Rbv;
	break;
      case 0x48: /* __EQV; */
	Rcv = Rav ^ ~Rbv;
	break;
      }
    break;

  case 0x13:
    switch(func11 & 0x7F)
      {
      case 0x00: /*__MULL; */
	Rcv = (Rav*Rbv) & 0x00000000FFFFFFFFllu;
	if(Rcv & BIT31) Rcv |= EXTND32;
	break;
      case 0x30: /*__UMULH; */
	mul128(Rav, Rbv, &tmp2, &tmp3);
	Rcv = tmp2;
	break;
      case 0x20: /*__MULQ; */
	mul128(Rav, Rbv, &tmp2, &tmp3);
	Rcv = tmp3;
	break;
	
      default:
	printf("Warning Code 0x13 defalult execution. func11 %x\n",
	       func11&0x7F);
	Rcv = 0;
      }
    break;

    
  case 0x12:
    switch(func11 & 0x7F)
      {
      case 0x39: /*__SLL; */
	Rcv=(DATA_TYPE)Rav << (Rbv & 0x3F);
	break;
      case 0x34: /*__SRL; */
	Rcv=(DATA_TYPE)Rav >> (Rbv & 0x3F);
	break;
      case 0x3c: /*__SRA; */
	Rcv=(long long)Rav >> (Rbv & 0x3F);
	break;
	
      case 0x06: /*__EXTBL; */
	Rcv = (DATA_TYPE)Rav >> ((Rbv & 0x7)*8);
	Rcv &= 0x00000000000000FFllu;
	break;
      case 0x16: /*__EXTWL; */
	Rcv = (DATA_TYPE)Rav >> ((Rbv & 0x7)*8);
	Rcv &= 0x000000000000FFFFllu;
	break;
      case 0x5a: /*__EXTWH; */
	Rcv = (DATA_TYPE)Rav << (64-(Rbv & 0x7)*8);
	Rcv &= 0x000000000000FFFFllu;
	break;
      case 0x26: /*__EXTLL; */
	Rcv = (DATA_TYPE)Rav >> ((Rbv & 0x7)*8);
	Rcv &= 0x00000000FFFFFFFFllu;
	break;
      case 0x6a: /*__EXTLH; */
	Rcv = (DATA_TYPE)Rav << (64-(Rbv & 0x7)*8);
	Rcv &= 0x00000000FFFFFFFFllu;
	break;
      case 0x36: /*__EXTQL; */
	Rcv = (DATA_TYPE)Rav >> ((Rbv & 0x7)*8);
	break;
      case 0x7a: /*__EXTQH; */
	Rcv = (DATA_TYPE)Rav << (64-(Rbv & 0x7)*8);
	break;
	
      case 0x0b: /*__INSBL; */
	byte_mask = 0x00000001ull << (Rbv & 0x7);
	byte_loc  = (Rbv & 0x7)*8;
	Rbv = Rav << (byte_loc & 0x3F);
	Rcv = zap(~byte_mask, Rbv);
	break;
      case 0x1b: /*__INSWL; */
	byte_mask = 0x00000003ull << (Rbv & 0x7);
	byte_loc  = (Rbv & 0x7)*8;
	Rbv = Rav << (byte_loc & 0x3F);
	Rcv = zap(~byte_mask, Rbv);
	break;
      case 0x57: /*__INSWH; */
	byte_mask = 0x00000003ull << (Rbv & 0x7);
	byte_loc  = 64 - (Rbv & 0x7)*8;
	Rbv = Rav >> (byte_loc & 0x3F);
	Rcv = zap(~(byte_mask >> 8), Rbv);
	break;
      case 0x2b: /*__INSLL; */
	byte_mask = 0x0000000Full << (Rbv & 0x7);
	byte_loc  = (Rbv & 0x7)*8;
	Rbv = Rav << (byte_loc & 0x3F);
	Rcv = zap(~byte_mask, Rbv);
	break;
      case 0x67: /*__INSLH; */
	byte_mask = 0x0000000Full << (Rbv & 0x7);
	byte_loc  = 64 - (Rbv & 0x7)*8;
	Rbv = Rav >> (byte_loc & 0x3F);
	Rcv = zap(~(byte_mask >> 8), Rbv);
	break;
      case 0x3b: /*__INSQL; */
	byte_mask = 0x000000FFull << (Rbv & 0x7);
	byte_loc  = (Rbv & 0x7)*8;
	Rbv = Rav << (byte_loc & 0x3F);
	Rcv = zap(~byte_mask, Rbv);
	break;
      case 0x77: /*__INSQH; */
	byte_mask = 0x000000FFull << (Rbv & 0x7);
	byte_loc  = 64 - (Rbv & 0x7)*8;
	Rbv = Rav >> (byte_loc & 0x3F);
	Rcv = zap(~(byte_mask >> 8), Rbv);
	break;
	
      case 0x02: /*__MSKBL; */
	Rbv = 0x00000001ull << (Rbv & 0x7);
	Rcv = zap(Rbv, Rav);
	break;
      case 0x12: /*__MSKWL; */
	Rbv = 0x00000003ull << (Rbv & 0x7);
	Rcv = zap(Rbv, Rav);
	break;
      case 0x52: /*__MSKWH; */
	Rbv = 0x00000003ull << (Rbv & 0x7);
	Rbv = Rbv >> 8;
	Rcv = zap(Rbv, Rav);
	break;
      case 0x22: /*__MSKLL; */
	Rbv = 0x0000000Full << (Rbv & 0x7);
	Rcv = zap(Rbv, Rav);
	break;
      case 0x62: /*__MSKLH; */
	Rbv = 0x0000000Full << (Rbv & 0x7);
	Rbv = Rbv >> 8;
	Rcv = zap(Rbv, Rav);
	break;
      case 0x32: /*__MSKQL; */
	Rbv = 0x000000FFull << (Rbv & 0x7);
	Rcv = zap(Rbv, Rav);
	break;
      case 0x72: /*__MSKQH; */
	Rbv = 0x000000FFull << (Rbv & 0x7);
	Rbv = Rbv >> 8;
	Rcv = zap(Rbv, Rav);
	break;
	
      case 0x30: /*__ZAP; */
	Rcv = zap(Rbv,  Rav);
	break;
      case 0x31: /*__ZAPNOT; */
	Rcv = zap(~Rbv, Rav);
	break;
	
      default:
	printf("Warning Code 0x12 defalult execution. func11 0x%u\n",
	       func11 & 0x7F);
	Rcv = 0;
      }
    break;

    
  case 0x16: /** Floating Point Codes **/
    {
      double Fav=0.0, Fbv=0.0, Fcv=0.0;
      memcpy(&Fav, &Rav, 8);
      memcpy(&Fbv, &Rbv, 8);
      
      switch(func11){
      case 0x0a0: /*__ADDT; */
	Fcv = Fav + Fbv;
	break;
      case 0x020:
      case 0x080: /*__ADDS; */
	Fcv = Fav + Fbv;
	break;
	
      case 0x081:
      case 0x0a1:
	Fcv = Fav - Fbv;
	break;
	
      case 0x0be: /*__CVTQT; */
	Fcv = (double)Rbv;
	break;
	
      case 0x0a2: /*__MULT; */
	Fcv = Fav * Fbv;
	break;
	
      case 0x0a3: /*__DIVT; */
	if(Fbv==0.0){
	  Fcv = 0.0;
	  break;
	}
	Fcv = (float)(Fav / Fbv);
	break;
	
      case 0x083: /*__DIVS; */
	if(Fbv==0.0){
	  Fcv = 0.0;
	  break;
	}
	Fcv = Fav / Fbv;
	break;
	
      case 0x5a5:
      case 0x0a5:/*__CMPTEQ; */
	if(Fav==Fbv) Fcv=2.0; else Fcv=0.0;
	break;
      case 0x0a7:/*__CMPTLE; */
	if(Fav<=Fbv) Fcv=2.0; else Fcv=0.0;
	break;
      case 0x0a6:/*__CMPTLT; */
	if(Fav< Fbv) Fcv=2.0; else Fcv=0.0;
	break;
	
      case 0x02f:/*__CVTTQ;  */
	tmp = (unsigned long long)Fbv;
	memcpy(&Fcv, &tmp, 8);
	break;
	
      default: printf("Warn 0x16 Floating Arithmetic func11 0x%x\n",
		      func11);
	Fcv = 0.0;
      }

      memcpy(&Rcv, &Fcv, 8);
      break;
    }
    
  case 0x17: /** Floating Point Codes **/
    {
      double Fav=0.0, Fbv=0.0, Fcv=0.0;
      memcpy(&Fav, &Rav, 8);
      memcpy(&Fbv, &Rbv, 8);
      
      switch(func11){
      case 0x020: /*__CPYS; */
	tmp  = Rav;
	tmp2 = Rbv;
	tmp2 = ((tmp2 << 1) >> 1); /* clear bit63 */
	if(tmp & BIT63) tmp2 |= BIT63;
	memcpy(&Fcv, &tmp2, 8);
	break;
      case 0x022: /*__CPYSE; */
	tmp  = Rav;
	tmp2 = Rbv;
	tmp  &= 0xFFF0000000000000llu; /* sign and Exp of Fra */
	tmp2 &= 0x000FFFFFFFFFFFFFllu; /* Fraction     of Frb */
	tmp |= tmp2;
	memcpy(&Fcv, &tmp, 8);
	break;
	
      case 0x010: /*__CVTLQ; */
	tmp = Rbv;
	tmp2 = (tmp >> 62) & 0x3;
	tmp2 = tmp2 << 30;
	tmp2 |= ((tmp >> 29) & 0x3FFFFFFFull);
	if(tmp2 & BIT31) tmp2 |= EXTND32;
	memcpy(&Fcv, &tmp2, 8);
	break;
	
      default:
	Fcv = 0.0;
	break;
      }
      memcpy(&Rcv, &Fcv, 8);
      break;
    }
  }

  return Rcv;
}
