/*
 * Instruction Definition of the Alpha AXP.
 * Defined and Written by Kenji Kise, 1999
 *******************************************************/

/***** Control Transfer instructions (int and float) *****/
#define __BR     1 /* Unconditional Branch */
#define __BSR    2 /* Branch to Subroutine */
#define __BEQ    3 /* Branch if Register Equal to Zero */
#define __BGE    4 /* Branch if Register Greater Than or Equal to Zero */
#define __BGT    5 /* Branch if Register Greater Than Zero */
#define __BLBC   6 /* Branch if Register Low Bit Is Clear */
#define __BLBS   7 /* Branch if Register Low Bit Is Set */
#define __BLE    8 /* Branch if Register Less Than or Equal to Zero */
#define __BLT    9 /* Branch if Register Less Than Zero */
#define __BNE   10 /* Branch if Register Not Equal to Zero */
#define __JMP   11 /* Jump */
#define __JSR   12
#define __RET   13 /* Return from Subroutine */
#define __JSR_COROUTINE 14 /* Jump to Subroutine Return */
#define __FBEQ  15
#define __FBGE  16
#define __FBGT  17
#define __FBLE  18
#define __FBLT  19
#define __FBNE  20
/***** Memory Access Instructions (int and float) *****/
#define __LDL   30 /* Load Sign-Extended Longword from Memory to Register */
#define __LDQ   31 /* Load Quadword from Memory to Register */
#define __LDQ_U 32 /* Load Unaligned Quadword from Memory to Register */
#define __LDL_L 33 /* Load Sign-Extended Longword from Memory to Register Locked*/
#define __LDQ_L 34 /* Load Quadword from Memory to Register Locked */
#define __STL_C 35 /* Store Longword from Register to Memory Conditional */
#define __STQ_C 36 /* Store Quadword from Register to Memory Conditional */
#define __STL   37 /* Store Longword from Register to Memory  */
#define __STQ   38 /* Store Quadword from Register to Memory  */
#define __STQ_U 39 /* Store Unaligned Quadword from Register to Memory  */
#define __LDF   41
#define __LDG   42
#define __LDS   43
#define __LDT   44
#define __STF   45
#define __STG   46
#define __STS   47
#define __STT   48
#define __FETCH   50
#define __FETCH_M 51
#define __MB      52
#define __RC      53
#define __RPCC    54
#define __RS      55
#define __TRAPB   56

/***** Arithmetic instructions (int and float)  *****/
/* code 0x10 */
#define __LDA    60 /* Load Address */
#define __LDAH   61 /* Load Address High */
#define __ADDL   62 /* Add Longword */
#define __ADDL_V 63 /* Add Longword */
#define __ADDQ   64 /* Add Quadword */
#define __ADDQ_V 65 /* Add Quadword */
#define __S4ADDL 66 /* Scaled Add Longword by 4 */
#define __S8ADDL 67 /* Scaled Add Longword by 8 */
#define __S4ADDQ 68 /* Scaled Add Quadword by 4 */
#define __S8ADDQ 69 /* Scaled Add Quadword by 8 */
#define __CMPEQ  70 /* Compare Signed Quadword Equal */
#define __CMPLE  71 /* Compare Signed Quadword Less Than or Equal */
#define __CMPLT  72 /* Compare Signed Quadword Less Than */
#define __CMPULE 73 /* Compare Unsigned Quadword Less Than or Equal */
#define __CMPULT 74 /* Compare Unsigned Quadword Less Than */
#define __CMPBGE 75 /* Compare Byte */
#define __SUBL   76 /* Subtract Longword */
#define __SUBL_V 77 /* Subtract Longword */
#define __S4SUBL 78 /* Scaled Subtract Longword by 4 */
#define __S8SUBL 79 /* Scaled Subtract Longword by 8 */
#define __SUBQ   80 /* Scaled Quadword */
#define __SUBQ_V 81 /* Scaled Quadword */
#define __S4SUBQ 82 /* Scaled Subtract Quadword by 4 */
#define __S8SUBQ 83 /* Scaled Subtract Quadword by 8 */
/* code 0x11 */
#define __AND    84 /* Logical Product */
#define __BIC    85 /* Logical Product with Complement */
#define __BIS    86 /* Logical Sum (OR) */
#define __EQV    87 /* Logical Equivalence (XORNOT) */
#define __ORNOT  88 /* Logical Sum with Complement */
#define __XOR    89 /* Logical Difference */
#define __CMOVEQ  90 /* CMOVE if Register Equal to Zero */
#define __CMOVGE  91 /* CMOVE if Register Greater than or Equal to Zero */
#define __CMOVGT  92 /* CMOVE if Register Greater than Zero */
#define __CMOVLBC 93 /* CMOVE if Register Low Bit Clear */
#define __CMOVLBS 94 /* CMOVE if Register Low Bit Set */
#define __CMOVLE  95 /* CMOVE if Register Less than or Equal to Zero */
#define __CMOVLT  96 /* CMOVE if Register Less than Zero */
#define __CMOVNE  97 /* CMOVE if Register Not Equal to Zero */

/* code 0x12 */
#define __SLL     98 /* Shift Left Logical */
#define __SRL     99 /* Shift Right Logical */
#define __SRA    100 /* Shift Right Arithmetic */
#define __EXTBL  101 /* Extract Byte Low */
#define __EXTWL  102 /* Extract Word Low */
#define __EXTLL  103 /* Extract Longword Low */
#define __EXTQL  104 /* Extract Quadword Low */
#define __EXTWH  105 /* Extract Word High */
#define __EXTLH  106 /* Extract Longword High */
#define __EXTQH  107 /* Extract Quadword High */
#define __INSBL  108 /* Insert Byte Low */
#define __INSWL  109 /* Insert Word Low */
#define __INSLL  110 /* Insert Longword Low */
#define __INSQL  111 /* Insert Quadword Low */
#define __INSWH  112 /* Insert Word High */
#define __INSLH  113 /* Insert Longword High */
#define __INSQH  114 /* Insert Quadword High */
#define __MSKBL  115 /* Mask Byte Low */
#define __MSKWL  116 /* Mask Word Low */
#define __MSKLL  117 /* Mask Longword Low */
#define __MSKQL  118 /* Mask Quadword Low */
#define __MSKWH  119 /* Mask Word High */
#define __MSKLH  120 /* Mask Longword High */
#define __MSKQH  121 /* Mask Quadword High */
#define __ZAP    122 /* Zero Byte */
#define __ZAPNOT 123 /* Zero Byte Not */
/* code 0x13 */
#define __MULL   124 /* Multiply Longword */
#define __MULL_V 125 /* Multiply Longword */
#define __MULQ   126 /* Multiply Quadword */
#define __MULQ_V 127 /* Multiply Quadword */
#define __UMULH  128
/*
 * PAL
 **********************************************************/
#define __HALT     130
#define __IMB      131
#define __CALLSYS  132
#define __PAL19    133
#define __PAL1B    134
#define __PAL1D    135
#define __PAL1E    136
#define __PAL1F    137
#define __CALLPAL  138
/*
 * Floating Point 
 **********************************************************/
/* code 0x17 */
#define __CPYS     140
#define __CPYSN    141
#define __CPYSE    142
#define __MF_FPCR  143
#define __MT_FPCR  144
#define __CVTQL_SV 145
#define __CVTLQ    146
#define __CVTQL    147
#define __CVTQL_V  148
#define __FCMOVEQ  149
#define __FCMOVLT  150
#define __FCMOVLE  151
#define __FCMOVNE  152
#define __FCMOVGE  153
#define __FCMOVGT  154
/* code 0x16 */
#define __ADDS     160
#define __ADDT     161
#define __CMPTEQ   162
#define __CMPTLT   163
#define __CMPTLE   164
#define __CMPTUN   165
#define __CVTQS    166
#define __CVTQT    167
#define __CVTTS    168
#define __DIVS     169
#define __DIVT     170
#define __MULS     171
#define __MULT     172
#define __SUBS     173
#define __SUBT     174
#define __CVTTQ    175
#define __LDBU     200
#define __STB      201
#define __STW      202
