typedef unsigned long long ULL;
typedef unsigned long long DATA_TYPE;
typedef unsigned long long ADDR_TYPE;

#define func26(x) (x & 0x3FFFFFF)
#define func11(x) ((x >> 5) & 0x7FF)
#define ra(x)     ((x >> 21) & 0x1F)
#define rb(x)     ((x >> 16) & 0x1F)
#define rc(x)     (x & 0x1F)
#define fa(x)     (((x >> 21) & 0x1F)+32)
#define fb(x)     (((x >> 16) & 0x1F)+32)
#define fc(x)     ((x & 0x1F)+32)
#define lit8(x)   ((x >> 13) & 0xFF)
#define code(x)   ((x >> 26) & 0x3F)

#define BIT0  0x0000000000000001ull
#define BIT1  0x0000000000000002ull
#define BIT2  0x0000000000000004ull
#define BIT3  0x0000000000000008ull
#define BIT4  0x0000000000000010ull
#define BIT5  0x0000000000000020ull
#define BIT6  0x0000000000000040ull
#define BIT7  0x0000000000000080ull
#define BIT8  0x0000000000000100ull
#define BIT9  0x0000000000000200ull
#define BIT10 0x0000000000000400ull
#define BIT11 0x0000000000000800ull
#define BIT12 0x0000000000001000ull
#define BIT15 0x0000000000008000ull
#define BIT20 0x100000ull
#define BIT31 0x80000000ull
#define BIT63 0x8000000000000000ull

#define EXTND16 0xffffffffffff0000ull
#define EXTND21 0xffffffffffe00000ull
#define EXTND32 0xffffffff00000000ull
#define MASK16  0xFFFFull
#define MASK21  0x1FFFFFull
#define MASK32  0x00000000FFFFFFFFull

#define ZAP0  0xFFFFFFFFFFFFFF00ull
#define ZAP1  0xFFFFFFFFFFFF00FFull
#define ZAP2  0xFFFFFFFFFF00FFFFull
#define ZAP3  0xFFFFFFFF00FFFFFFull
#define ZAP4  0xFFFFFF00FFFFFFFFull
#define ZAP5  0xFFFF00FFFFFFFFFFull
#define ZAP6  0xFF00FFFFFFFFFFFFull
#define ZAP7  0x00FFFFFFFFFFFFFFull

/**** For get_iclass in decoder.c ***/
#define EX_OPM   1 /* Optimize Memory, Prefetch etc.           */
#define EX_PAL   2 /* PAL code.                                */
#define EX_JMP   3 /* JUMP, Register Addressing Branch         */
#define EX_BRA   4 /* Unconditional Branch                     */
#define EX_CBI   5 /* Conditional Branch Integer               */
#define EX_CBF   6 /* Conditional Branch Floating              */
#define EX_ARI   7 /* Logic and Arithmetic                     */
#define EX_ADR   8 /* Address calculation , NOT load and store */
#define EX_LDI   9 /* Load to Integer Reg                      */
#define EX_LDF  10 /* Load to Floating Reg                     */
#define EX_STI  11 /* Store Integer Value                      */
#define EX_STF  12 /* Store Floating Value                     */
#define EX_FLO  13 /* Floating Arithmetic and Logic            */
#define EX_CMI  14 /* Conditional Move Int.                    */
#define EX_CMF  15 /* Conditional Move Float.                  */

#define MEM_PAGE_SIZE  1024*1024 /* 1 Mbyte */
#define PAGE_MASK        0xFFFFF
#define PAGE_BIT              20
#define TLB_ENTRY            512
#define PAGE_SIZE           8192 /* 8KB page size */

/************************************************************/
typedef class architecture_state{ 
public:
  ADDR_TYPE pc;    /* program counter 64-bit width.       */
  DATA_TYPE r[32]; /* general purpose 64-bit integer Regs */
  DATA_TYPE f[32]; /* 64-bit width floating point Regs    */
} ARCH_STATE;

struct machine_setting{
  ULL current_argp_flag;
  int end_flag;              /* If set, the simulation ends. */
  int debug_flag;
  int distrib_flag;
  int syscall_flag;
  int print_state_flag;
  int control_trace_flag;
  int memory_trace_flag;
  int state_trace_flag;
  int state_trace_interval;
  ULL trace_start;
  int dcache_entry_bit;
  int dcache_line_bit;
  int verbose_interval;
  ULL end_tc;
};
struct evaluate{
  int used_memory;
  ULL retired_inst;
};

class instruction{
public:
  int cache_flush;       /* set if syscall chenges the memory */
  ULL tag;               /* Tag,  asigned in IF stage         */
  ADDR_TYPE cpc;         /* PC of the instruction             */
  ADDR_TYPE t_pc;        /* Calculated True Branch Target PC  */
  unsigned int ir;       /* 32bit instruction code            */
  int iclass;            /* the class of the instruction      */
  int LR1, LR2, LR3, LW; /* Logical Reg No.                   */
  ULL R1, R2, R3, W;     /* Renamed (phisical) Reg No.        */
  int isort;             /* the sort of the instruction       */
  int mem_byte;          /* Used for LD/ST instruction        */
  ADDR_TYPE Adr;         /* LD/ST Memory Address              */
  DATA_TYPE Rav;         /* The value of Ra */
  DATA_TYPE Rbv;         /* The value of Rb */
  DATA_TYPE Rmv;         /* The value of Rc */
  DATA_TYPE Rcv;         /* The value of Rc */
  /***** Methods *****/
  instruction();
  void Fetch(unsigned long long pc);
  void Decode();
  void Rename();
  void RegisterRead();
  void Execute();
  void Memory();
  void WriteBack();
};


/* Header File to define the Alpha object code format 
 * DIGITAL Unix (Alpha AXP), Arranged by Kenji Kise 1999.5
 *
 * Alpha AXP Type Definitions
 *  char 1byte
 *  int  4byte
 *  long 8byte
 *  long long 8byte
 *  float 4
 *  double 8
 *  short 2byte
 *****************************************************/
struct FF_filehdr {
  unsigned short f_magic;
  unsigned short f_nscns; /* use this */
  int		 f_timdat;	
  long long	 f_symptr;	
  int		 f_nsyms;	
  unsigned short f_opthdr;	
  unsigned short f_flags;
};

struct FF_aouthdr {
  short	magic;	  
  short	vstamp;	  
  short   bldrev;
  short   padcell;
  long long tsize;	/* use this */
  long long dsize;	/* use this */
  long long bsize;	
  long long entry;	/* use this */
  long long text_start;	
  long long data_start;	
  long long bbs_start;
  int	gprmask;	
  int	fprmask; 	
  long long	gp_value;
};

struct FF_scnhdr {
  char        s_name[8]; /* use this */
  long long   s_paddr;   /* use this */
  long long   s_vaddr;
  long long   s_size;    /* use this */
  long long   s_scnptr;  /* use this */
  long long   s_relptr; 
  long long   s_lnnoptr;
  unsigned short s_nreloc;
  unsigned short s_nlnno; 
  int     	s_flags;  
};

/************************************************************/
struct tlb_entry{
    int valid;
    unsigned long long adr;
    char *ptr;
};

class Imemory{
  unsigned int *ibuffer;
  ADDR_TYPE ibuffer_min_adr;  // minimun address of Ibuffer
  ADDR_TYPE ibuffer_max_adr;  // maximun address of Ibuffer
public:
  void Imemory_init(int size, ADDR_TYPE, ADDR_TYPE);
  unsigned int instruction_load(ADDR_TYPE);
  void         instruction_store(ADDR_TYPE, unsigned int data);
};

class Dmemory{
  struct tlb_entry tlb[TLB_ENTRY];
  DATA_TYPE sbuffer_min; /* Minimun Address of Stack  */
  ADDR_TYPE sbuffer_max; /* Maximun Address of Stack  */
  ADDR_TYPE dbuffer_min; /* Minimun Address of Data   */
  ADDR_TYPE dbuffer_max; /* Maximun Address of Data   */
public:
  char *adr_translate(unsigned long long adr);
  void set_dbuffer(ADDR_TYPE);
  void set_dbuffer(ADDR_TYPE, ADDR_TYPE);
  void set_sbuffer(ADDR_TYPE, ADDR_TYPE);
  
  char         load_byte( ADDR_TYPE);
  unsigned int load_4byte(ADDR_TYPE);
  DATA_TYPE    load_8byte(ADDR_TYPE);

  void store_byte( ADDR_TYPE, char);
  void store_4byte(ADDR_TYPE, unsigned int);
  void store_8byte(ADDR_TYPE, DATA_TYPE);

  DATA_TYPE load(int byte, ADDR_TYPE);
  void store(int byte, ADDR_TYPE, DATA_TYPE);
};


/**** Extern Functions ****/
extern void debugmode();
extern void mem_init(struct architecture_state *,
		     int, char **, char **);
extern ULL get_hex(char *);
extern int get_iclass(DATA_TYPE);
extern int get_isort(unsigned int ir);
extern DATA_TYPE get_store_data(int, DATA_TYPE);
extern DATA_TYPE get_load_data(int, DATA_TYPE);
extern DATA_TYPE execute_branch(unsigned long long *pc,
				unsigned long long diff21);
extern DATA_TYPE ex_arith(int code,
			  unsigned int func11,
			  DATA_TYPE Rav,
			  DATA_TYPE Rbv);
extern DATA_TYPE ex_cmov(int code,
			 unsigned int func11,
			 DATA_TYPE Rav,
			 DATA_TYPE Rbv,
			 DATA_TYPE Rmv);
extern DATA_TYPE execute_jmp(ADDR_TYPE *pc, DATA_TYPE Rbv);
extern DATA_TYPE calculate_adr(int code, DATA_TYPE Rbv,
			       DATA_TYPE diff16);
extern void execute_pal(instruction *);
extern void ex_branch(int, ADDR_TYPE*, DATA_TYPE, DATA_TYPE);
