/* Decode Instruction and Get Assembler Strings.
 * get_assem_a for the Alpha AXP. Written by Kenji Kise, 1999.
 *******************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#include "define.h"
#include "code.h"

/*****
 * There are four instruction types;
 *  (1) PAL Code Type ; Code<31:26>            Function_code<25:0>            ;
 *  (2) Branch Type   ; Code<31:26> Ra<25:21>                  Diff<20:0>     ;
 *  (3) Memory Type   ; Code<31:26> Ra<25:21> Rb<20:16>        Diff<15:0>     ;
 *  (4) Operation Type; Code<31:26> Ra<25:21> Rb<20:16> Function<15:5> Rc<4:0>;
 *****/

char buff[256];

/*
 *
 **********************************************************/
char *get_assem_a(unsigned int ir, unsigned long long pc){
  unsigned int code;
  unsigned int ra,rb,rc;
  unsigned int func26, func11;
  unsigned long long diff21, diff16;
  unsigned int lit8;                
  char temp[256];
  
  func26 =  ir & 0x3FFFFFF;      /* ir<25:0>  */
  func11 = (ir >> 5) & 0x7FF;    /* ir<15:5>  */
  code   = ir >> 26;             /* ir<31:26> */
  diff21 = ir & 0x1FFFFF;        /* ir<21:0>  */
  diff16 = ir &   0xFFFF;        /* ir<15:0>  */
  ra     = (ir >> 21) & 0x1F;    /* ir<25:21> */
  rb     = (ir >> 16) & 0x1F;    /* ir<20:16> */
  rc     =  ir        & 0x1F;    /* ir<4:0>   */
  lit8   = (ir >> 13) & 0xFF;    /* ir<20:13> */

  if(diff21 & BIT20) diff21 |= EXTND21; /* signed extended */
  if(diff16 & BIT15) diff16 |= EXTND16; /* signed extended */
	 
  switch(code)
    {
      /***** Memory Type Without Function Code *****/
    case 0x0a: sprintf(buff, "ldub   r%d,(r%d)",  ra, rb); break;
    case 0x0e: sprintf(buff, "stb    r%d,(r%d)",  ra, rb); break;
      
    case 0x08: sprintf(buff, "lda    r%d,(r%d)",  ra, rb); break;
    case 0x09: sprintf(buff, "ldah   r%d,(r%d)",  ra, rb); break;
    case 0x28: sprintf(buff, "ldl    r%d,(r%d)",  ra, rb); break;
    case 0x29: sprintf(buff, "ldq    r%d,(r%d)",  ra, rb); break;
    case 0x0b: sprintf(buff, "ldq_u  r%d,(r%d)",  ra, rb); break;
    case 0x2a: sprintf(buff, "ldl_l  r%d,(r%d)",  ra, rb); break;
    case 0x2b: sprintf(buff, "ldq_l  r%d,(r%d)",  ra, rb); break;

    case 0x2e: sprintf(buff, "stl_c  r%d,(r%d)",  ra, rb); break;
    case 0x2f: sprintf(buff, "stq_c  r%d,(r%d)",  ra, rb); break;
    case 0x2c: sprintf(buff, "stl    r%d,(r%d)",  ra, rb); break;
    case 0x2d: sprintf(buff, "stq    r%d,(r%d)",  ra, rb); break;
    case 0x0f: sprintf(buff, "stq_u  r%d,(r%d)",  ra, rb); break;

    case 0x20: sprintf(buff, "ldf  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x21: sprintf(buff, "ldg  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x22: sprintf(buff, "lds  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x23: sprintf(buff, "ldt  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x24: sprintf(buff, "stf  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x25: sprintf(buff, "stg  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x26: sprintf(buff, "sts  f%d,%qd(r%d)",   ra, diff16, rb); break;
    case 0x27: sprintf(buff, "stt  f%d,%qd(r%d)",   ra, diff16, rb); break;
      
      /***** Memory Type With Function Code *****/
    case 0x18:
      switch(diff16){
      case 0x8000: sprintf(buff, "fetch r%d, r%d",  ra,rb); break;
      case 0xe000: sprintf(buff, "rc r%d, r%d",     ra,rb); break;
      case 0x0000: sprintf(buff, "trapb r%d, r%d",  ra,rb); break;
      case 0xa000: sprintf(buff, "fetch_m r%d, r%d",ra,rb); break;
      case 0xc000: sprintf(buff, "rpcc r%d, r%d",   ra,rb); break;
      case 0x4000: sprintf(buff, "mb r%d, r%d",     ra,rb); break;
      case 0xf000: sprintf(buff, "rs r%d, r%d",     ra,rb); break;
      default:
	printf("Warning In Memory Type With Function Code.\n");
	sprintf(buff, "n/a");
      }
      break;
      
      /***** branch instruction *****/
    case 0x1a:
      switch((diff16 >> 14) & 3){
      case 0: sprintf(buff, "jmp  r%d, r%d",ra,rb); break;
      case 1: sprintf(buff, "jsr  r%d, r%d",ra,rb); break;
      case 3: sprintf(buff, "jsr_coroutine  r%d, r%d",ra,rb); break;
      case 2: sprintf(buff, "ret  r%d, r%d",ra,rb); break;
      default:
	printf("Warning In Branch Type With Function Code.\n");
	sprintf(buff, "n/a");
      } break;
      
    case 0x30: sprintf(buff, "br   r%d, 0x%qx", ra, pc+diff21*4); break;
    case 0x34: sprintf(buff, "bsr  r%d, 0x%qx",ra, pc+diff21*4); break;
      
    case 0x39: sprintf(buff, "beq  r%d, 0x%qx",ra, pc+diff21*4); break;
    case 0x3e: sprintf(buff, "bge  r%d, 0x%qx",ra, pc+diff21*4); break;
    case 0x3f: sprintf(buff, "bgt  r%d, 0x%qx",ra, pc+diff21*4); break;
    case 0x38: sprintf(buff, "blbc r%d, 0x%qx",ra,pc+diff21*4); break;
    case 0x3c: sprintf(buff, "blbs r%d, 0x%qx",ra,pc+diff21*4); break;
    case 0x3b: sprintf(buff, "ble  r%d, 0x%qx",ra, pc+diff21*4); break;
    case 0x3a: sprintf(buff, "blt  r%d, 0x%qx",ra, pc+diff21*4); break;
    case 0x3d: sprintf(buff, "bne  r%d, 0x%qx",ra, pc+diff21*4); break;
      
    case 0x31: sprintf(buff, "fbeq  r%d, 0x%qx 0x%qx",ra,pc+4,diff21);break;
    case 0x35: sprintf(buff, "fbne  r%d, 0x%qx 0x%qx",ra,pc+4,diff21);break;
    case 0x32: sprintf(buff, "fblt  r%d, 0x%qx 0x%qx",ra,pc+4,diff21);break;
    case 0x36: sprintf(buff, "fbge  r%d, 0x%qx 0x%qx",ra,pc+4,diff21);break;
    case 0x33: sprintf(buff, "fble  r%d, 0x%qx 0x%qx",ra,pc+4,diff21);break;
    case 0x37: sprintf(buff, "fbgt  r%d, 0x%qx 0x%qx",ra,pc+4,diff21);break;
      
    case 0x10:
      switch(func11 & 0x7F)
	{
	case 0x00: strcpy(buff, "addl  "); break;
	case 0x40: strcpy(buff, "addl/v"); break;
	case 0x20: strcpy(buff, "addq  "); break;
	case 0x60: strcpy(buff, "addq/v"); break;
	case 0x3d: strcpy(buff, "cmpule"); break;
	case 0x0f: strcpy(buff, "cmpbge"); break;
	case 0x09: strcpy(buff, "subl  "); break;
	case 0x49: strcpy(buff, "subl/v"); break;
	case 0x29: strcpy(buff, "subq  "); break;
	case 0x69: strcpy(buff, "subq/v"); break;
	case 0x2d: strcpy(buff, "cmpeq "); break;
	case 0x4d: strcpy(buff, "cmplt "); break;
	case 0x6d: strcpy(buff, "cmple "); break;
	case 0x1d: strcpy(buff, "cmpult"); break;
	case 0x02: strcpy(buff, "s4addl"); break;
	case 0x22: strcpy(buff, "s4addq"); break;
	case 0x0b: strcpy(buff, "s4subl"); break;
	case 0x2b: strcpy(buff, "s4subq"); break;
	case 0x12: strcpy(buff, "s8addl"); break;
	case 0x32: strcpy(buff, "s8addq"); break;
	case 0x1b: strcpy(buff, "s8subl"); break;
	case 0x3b: strcpy(buff, "s8subq"); break;
	default:
	  printf("Error in 0x10 arithmetic\n");
	  sprintf(buff, "n/a");
	}
      if(ir & BIT12) sprintf(temp, " r%d,0x%x,r%d",ra,lit8,rc); 
      else           sprintf(temp, " r%d,r%d,r%d", ra,rb,rc);
      strcat(buff, temp);
      break;
      
    case 0x11:
      switch(func11 & 0x7F)
	{
	case 0x00: strcpy(buff, "and"); break;
	case 0x20: strcpy(buff, "bis"); break;
	case 0x40: strcpy(buff, "xor"); break;
	case 0x08: strcpy(buff, "bic"); break;
	case 0x28: strcpy(buff, "ornot"); break;
	case 0x48: strcpy(buff, "eqv"); break;

	case 0x24: strcpy(buff, "cmoveq"); break;
	case 0x46: strcpy(buff, "cmovge"); break;
	case 0x66: strcpy(buff, "cmovgt"); break;
	case 0x16: strcpy(buff, "cmovlgc"); break;
	case 0x14: strcpy(buff, "cmovlbs"); break;
	case 0x64: strcpy(buff, "cmovle"); break;
	case 0x44: strcpy(buff, "cmovlt"); break;
	case 0x26: strcpy(buff, "cmovne"); break;
	default:
	  printf("Error in 0x11 arithmetic\n");
	  sprintf(buff, "n/a");
	}
      if(ir & BIT12) sprintf(temp, "  r%d,0x%x,r%d",ra,lit8,rc); 
      else           sprintf(temp, "  r%d,r%d,r%d", ra,rb,rc);
      strcat(buff, temp);
      break;
      
    case 0x12:
      switch(func11 & 0x7F)
	{
	case 0x39: strcpy(buff, "sll"); break;
	case 0x34: strcpy(buff, "srl"); break;
	case 0x3c: strcpy(buff, "sra"); break;
	case 0x06: strcpy(buff, "extbl"); break;
	case 0x16: strcpy(buff, "extwl"); break;
	case 0x26: strcpy(buff, "extll"); break;
	case 0x36: strcpy(buff, "extql"); break;
	case 0x5a: strcpy(buff, "extwh"); break;
	case 0x6a: strcpy(buff, "extlh"); break;
	case 0x7a: strcpy(buff, "extqh"); break;
	case 0x0b: strcpy(buff, "insbl"); break;
	case 0x1b: strcpy(buff, "inswl"); break;
	case 0x2b: strcpy(buff, "insll"); break;
	case 0x3b: strcpy(buff, "insql"); break;
	case 0x57: strcpy(buff, "inswh"); break;
	case 0x67: strcpy(buff, "inslh"); break;
	case 0x77: strcpy(buff, "insqh"); break;
	case 0x02: strcpy(buff, "mskbl"); break;
	case 0x12: strcpy(buff, "mskwl"); break;
	case 0x22: strcpy(buff, "mskll"); break;
	case 0x32: strcpy(buff, "mskql"); break;
	case 0x52: strcpy(buff, "mskwh"); break;
	case 0x62: strcpy(buff, "msklh"); break;
	case 0x72: strcpy(buff, "mskqh"); break;
	case 0x30: strcpy(buff, "zap"); break;
	case 0x31: strcpy(buff, "zapnot"); break;
	default:
	  printf("Warning in 0x12, func11 0x%x arithmetic\n",func11);
	  sprintf(buff, "n/a");
	}
      if(ir & BIT12) sprintf(temp, "  r%d,0x%x,r%d",ra,lit8,rc); 
      else           sprintf(temp, "  r%d,r%d,r%d", ra,rb,rc);
      strcat(buff, temp);
      break;

    case 0x13:
      switch(func11 & 0x7F)
	{
	case 0x00: sprintf(buff, "mull  r%d,r%d,r%d",   ra,rb,rc); break;
	case 0x60: sprintf(buff, "mulq/v  r%d,r%d,r%d", ra,rb,rc); break;
	case 0x40: sprintf(buff, "mull/v  r%d,r%d,r%d", ra,rb,rc); break;
	case 0x30: sprintf(buff, "umulh  r%d,r%d,r%d",  ra,rb,rc); break;
	case 0x20: sprintf(buff, "mulq  r%d,r%d,r%d",   ra,rb,rc); break;
	default:
	  printf("Warning in 0x13 arithmetic\n");
	  sprintf(buff, "n/a");
	}
      break;

      /***** PAL Code *****/
    case 0x00:
      switch(func26 & 0xffff){
      case 0x0000: sprintf(buff, "halt"); break;
      case 0x0086: sprintf(buff, "imb"); break;
      case 0x0083: sprintf(buff, "callsys"); break;
      default:
	sprintf(buff, "call_pal  0x%x", func26);
      } break;
      
    case 0x19: sprintf(buff, "pal19"); break;
    case 0x1f: sprintf(buff, "pal1f"); break;
    case 0x1b: sprintf(buff, "pal1b"); break;
    case 0x1d: sprintf(buff, "pal1d"); break;
    case 0x1e: sprintf(buff, "pal1e"); break;

      /***** Floating Arithmetic *****/
    case 0x16:
      switch(func11){
      case 0x080: strcpy(buff, "adds"); break;
      case 0x000: strcpy(buff, "adds/c"); break;
      case 0x040: strcpy(buff, "adds/m"); break;
      case 0x0c0: strcpy(buff, "adds/d"); break;
      case 0x180: strcpy(buff, "adds/u"); break;
      case 0x100: strcpy(buff, "adds/uc"); break;
      case 0x140: strcpy(buff, "adds/um"); break;
      case 0x1c0: strcpy(buff, "adds/ud"); break;

      case 0x0a0: strcpy(buff, "addt"); break;
      case 0x0a1: strcpy(buff, "subt"); break;
      case 0x020: strcpy(buff, "addt/c"); break;
      case 0x060: strcpy(buff, "addt/m"); break;
      case 0x0e0: strcpy(buff, "addt/d"); break;
      case 0x1a0: strcpy(buff, "addt/u"); break;
      case 0x120: strcpy(buff, "addt/uc"); break;
      case 0x160: strcpy(buff, "addt/um"); break;
      case 0x1e0: strcpy(buff, "addt/ud"); break;

      case 0x0a5: strcpy(buff, "cmpteq"); break;
      case 0x0a6: strcpy(buff, "cmptlt"); break;
      case 0x0a7: strcpy(buff, "cmptle"); break;
      case 0x0a4: strcpy(buff, "cmptun"); break;
      case 0x0bc:
      case 0x03c:
      case 0x07c:
      case 0x0fc:
	strcpy(buff, "cvtqs"); break;
      case 0x0be:
      case 0x03e:
      case 0x07e:
      case 0x0fe:
	strcpy(buff, "cvtqt"); break;
      case 0x083: strcpy(buff, "divs"); break;
      case 0x0a3: strcpy(buff, "divt"); break;
	
      default:
	sprintf(buff, "Floating Arithmetic func11 0x%x",func11);
	break;
      }
      sprintf(temp, "  f%d,f%d,f%d", ra,rb,rc);
      strcat(buff, temp);
      break;

    case 0x17:
      switch(func11){
      case 0x020: strcpy(buff, "cpys"); break;
      case 0x021: strcpy(buff, "cpysn"); break;
      case 0x022: strcpy(buff, "cpyse"); break;
      case 0x025: strcpy(buff, "mf_fpcr"); break;
      case 0x024: strcpy(buff, "mt_fpcr"); break;
      case 0x530: strcpy(buff, "cvtql/sv"); break;
      case 0x010: strcpy(buff, "cvtlq"); break;
      case 0x030: strcpy(buff, "cvtql"); break;
      case 0x130: strcpy(buff, "cvtql/v"); break;
      case 0x02a: strcpy(buff, "fcmoveq"); break;
      case 0x02c: strcpy(buff, "fcmovlt"); break;
      case 0x02e: strcpy(buff, "fcmovle"); break;
      case 0x02b: strcpy(buff, "fcmovne"); break;
      case 0x02d: strcpy(buff, "fcmovge"); break;
      case 0x02f: strcpy(buff, "fcmovgt"); break;

      default: sprintf(buff, "Code 0x17 Floating Arithmetic func11 0x%x",
		       func11);
	break;
      }
      break;
      
    default:
      printf("Warning Code 0x%x Default case in decode.c", code);
      sprintf(buff,"n/a");
    }
  return buff;
}

/*
 *
 **********************************************************/
int get_isort(unsigned int ir){
  unsigned int code;
  unsigned int func26, func11;
  unsigned long long diff16;
  int isort=255;
  
  code   = ir >> 26;             /* ir<31:26> */
  diff16 = ir &   0xFFFF;        /* ir<15:0>  */
  func11 = (ir >> 5) & 0x7FF;    /* ir<15:5>  */
  func26 =  ir & 0x3FFFFFF;      /* ir<25:0>  */

  if(diff16 & BIT15) diff16 |= EXTND16; /* signed extended */
	 
  switch(code)
    {
      /***** Memory Type Without Function Code *****/
    case 0x0a: isort = __LDBU; break;
    case 0x0e: isort = __STB; break;
    case 0x0d: isort = __STW; break;
      
    case 0x08: isort = __LDA; break;
    case 0x09: isort = __LDAH; break;
    case 0x28: isort = __LDL; break;
    case 0x29: isort = __LDQ; break;
    case 0x0b: isort = __LDQ_U; break;
    case 0x2a: isort = __LDL_L; break;
    case 0x2b: isort = __LDQ_L; break;
    case 0x2e: isort = __STL_C; break;
    case 0x2f: isort = __STQ_C; break;
    case 0x2c: isort = __STL; break;
    case 0x2d: isort = __STQ; break;
    case 0x0f: isort = __STQ_U; break;

    case 0x20: isort=__LDF; break;
    case 0x21: isort=__LDQ; break;
    case 0x22: isort=__LDS; break;
    case 0x23: isort=__LDT; break;
    case 0x24: isort=__STF; break;
    case 0x25: isort=__STQ; break;
    case 0x26: isort=__STS; break;
    case 0x27: isort=__STT; break;
      
      /***** Memory Type With Function Code *****/
    case 0x18:
      switch(diff16){
      case 0x8000: isort=__FETCH;  break;
      case 0xe000: isort=__RC;     break;
      case 0x0000: isort=__TRAPB;  break;
      case 0xa000: isort=__FETCH_M;break;
      case 0xc000: isort=__RPCC;   break;
      case 0x4000: isort=__MB;     break;
      case 0xf000: isort=__RS;     break;
      }
      break;
      
      /***** branch instruction *****/
    case 0x1a:
      switch((diff16 >> 14) & 3){
      case 0: isort=__JMP; break;
      case 1: isort=__JSR; break;
      case 3: isort=__JSR_COROUTINE; break;
      case 2: isort=__RET; break;
      } break;
      
    case 0x30: isort=__BR;  break;
    case 0x34: isort=__BSR; break;
    case 0x39: isort=__BEQ; break;
    case 0x3e: isort=__BGE; break;
    case 0x3f: isort=__BGT; break;
    case 0x38: isort=__BLBC;break;
    case 0x3c: isort=__BLBS;break;
    case 0x3b: isort=__BLE; break;
    case 0x3a: isort=__BLT; break;
    case 0x3d: isort=__BNE; break;
    case 0x31: isort=__FBEQ; break;
    case 0x35: isort=__FBNE; break;
    case 0x32: isort=__FBLT; break;
    case 0x36: isort=__FBGE; break;
    case 0x33: isort=__FBLE; break;
    case 0x37: isort=__FBGT; break;
      
    case 0x10:
      switch(func11 & 0x7F)
	{
	case 0x00: isort=__ADDL;   break;
	case 0x40: isort=__ADDL_V; break;
	case 0x20: isort=__ADDQ;   break;
	case 0x60: isort=__ADDQ_V; break;
	case 0x3d: isort=__CMPULE; break;
	case 0x0f: isort=__CMPBGE; break;
	case 0x09: isort=__SUBL;   break;
	case 0x49: isort=__SUBL_V; break;
	case 0x29: isort=__SUBQ;   break;
	case 0x69: isort=__SUBQ_V; break;
	case 0x2d: isort=__CMPEQ;  break;
	case 0x4d: isort=__CMPLT;  break;
	case 0x6d: isort=__CMPLE;  break;
	case 0x1d: isort=__CMPULT; break;
	case 0x02: isort=__S4ADDL; break;
	case 0x22: isort=__S4ADDQ; break;
	case 0x0b: isort=__S4SUBL; break;
	case 0x2b: isort=__S4SUBQ; break;
	case 0x12: isort=__S8ADDL; break;
	case 0x32: isort=__S8ADDQ; break;
	case 0x1b: isort=__S8SUBL; break;
	case 0x3b: isort=__S8SUBQ; break;
	}
      break;
      
    case 0x11:
      switch(func11 & 0x7F)
	{
	case 0x00: isort=__AND;   break;
	case 0x20: isort=__BIS;   break;
	case 0x40: isort=__XOR;   break;
	case 0x08: isort=__BIC;   break;
	case 0x28: isort=__ORNOT; break;
	case 0x48: isort=__EQV;   break;

	case 0x24: isort=__CMOVEQ; break;
	case 0x26: isort=__CMOVNE; break;
	case 0x14: isort=__CMOVLBS; break;
	case 0x44: isort=__CMOVLT; break;
	case 0x46: isort=__CMOVGE; break;
	case 0x16: isort=__CMOVLBC; break;
	case 0x64: isort=__CMOVLE; break;
	case 0x66: isort=__CMOVGT; break;
	}
      break;
      
    case 0x12:
      switch(func11 & 0x7F)
	{
	case 0x39: isort=__SLL; break;
	case 0x34: isort=__SRL; break;
	case 0x3c: isort=__SRA; break;
	case 0x06: isort=__EXTBL; break;
	case 0x16: isort=__EXTWL; break;
	case 0x26: isort=__EXTLL; break;
	case 0x36: isort=__EXTQL; break;
	case 0x5a: isort=__EXTWH; break;
	case 0x6a: isort=__EXTLH; break;
	case 0x7a: isort=__EXTQH; break;
	case 0x0b: isort=__INSBL; break;
	case 0x1b: isort=__INSWL; break;
	case 0x2b: isort=__INSLL; break;
	case 0x3b: isort=__INSQL; break;
	case 0x57: isort=__INSWH; break;
	case 0x67: isort=__INSLH; break;
	case 0x77: isort=__INSQH; break;
	case 0x02: isort=__MSKBL; break;
	case 0x12: isort=__MSKWL; break;
	case 0x22: isort=__MSKLL; break;
	case 0x32: isort=__MSKQL; break;
	case 0x52: isort=__MSKWH; break;
	case 0x62: isort=__MSKLH; break;
	case 0x72: isort=__MSKQH; break;
	case 0x30: isort=__ZAP; break;
	case 0x31: isort=__ZAPNOT; break;
	}
      break;

    case 0x13:
      switch(func11 & 0x7F)
	{
	case 0x00: isort=__MULL;   break;
	case 0x60: isort=__MULQ_V; break;
	case 0x40: isort=__MULL_V; break;
	case 0x30: isort=__UMULH;  break;
	case 0x20: isort=__MULQ;   break;
	}
      break;

      /***** PAL Code *****/
    case 0x00:
      switch(func26 & 0xffff){
      case 0x0000: isort=__HALT; break;
      case 0x0086: isort=__IMB; break;
      case 0x0083: isort=__CALLSYS; break;
      default: isort=__CALLPAL;
      } break;
      
    case 0x19: sprintf(buff, "pal19"); break;
    case 0x1f: sprintf(buff, "pal1f"); break;
    case 0x1b: sprintf(buff, "pal1b"); break;
    case 0x1d: sprintf(buff, "pal1d"); break;
    case 0x1e: sprintf(buff, "pal1e"); break;

      /***** Floating Arithmetic *****/
    case 0x16:
      switch(func11){
      case 0x080:
      case 0x000:
      case 0x040:
      case 0x0c0:
      case 0x180:
      case 0x100:
      case 0x140:
      case 0x1c0: isort=__ADDS; break;

      case 0x0a0:
      case 0x020:
      case 0x060:
      case 0x0e0:
      case 0x1a0:
      case 0x120:
      case 0x160:
      case 0x1e0: isort=__ADDT; break;

      case 0x081: isort=__SUBS; break;
      case 0x0a1: isort=__SUBT; break;

      case 0x0ac: isort=__CVTTS; break;
	
      case 0x5a5:
      case 0x0a5: isort=__CMPTEQ; break;
      case 0x0a6: isort=__CMPTLT; break;
      case 0x0a7: isort=__CMPTLE; break;
      case 0x0a4: isort=__CMPTUN; break;
      case 0x0bc:
      case 0x03c:
      case 0x07c:
      case 0x0fc: isort=__CVTQS; break;
      case 0x0be:
      case 0x03e:
      case 0x07e:
      case 0x0fe: isort=__CVTQT; break;
      case 0x083: isort=__DIVS; break;
      case 0x0a3: isort=__DIVT; break;
      case 0x082: isort=__MULS; break;
      case 0x0a2: isort=__MULT; break;
      case 0x02f:
      case 0x0af: isort=__CVTTQ;
	
      default: sprintf(buff, "Floating Arithmetic func11 0x%x",func11);
	break;
      }
      break;

    case 0x17:
      switch(func11){
      case 0x020: isort=__CPYS; break;
      case 0x021: isort=__CPYSN; break;
      case 0x022: isort=__CPYSE; break;
      case 0x025: isort=__MF_FPCR; break;
      case 0x024: isort=__MT_FPCR; break;
      case 0x530: isort=__CVTQL_SV; break;
      case 0x010: isort=__CVTLQ; break;
      case 0x030: isort=__CVTQL; break;
      case 0x130: isort=__CVTQL_V; break;
      case 0x02a: isort=__FCMOVEQ; break;
      case 0x02c: isort=__FCMOVLT; break;
      case 0x02e: isort=__FCMOVLE; break;
      case 0x02b: isort=__FCMOVNE; break;
      case 0x02d: isort=__FCMOVGE; break;
      case 0x02f: isort=__FCMOVGT; break;
      }
      break;
    }
  
  //  if(isort==255){
  //    printf("In getassem.c default isort. code:0x%x, func11:0x%x.\n",
  //	   code, func11);
  //  }
  return isort;
}

