/* instruction.cc       This is a part of SimuAlpha Version 2.2a
 *                      Copyright (c) 1999,2000,2001, Kenji KISE
 ***************************************************************/
#include <iostream.h>
#include <string.h>
#include "define.h"

extern Dmemory Dmem;
extern Imemory Imem;
extern class architecture_state as;

DATA_TYPE diff21(unsigned int ir){
  long long diff21;
  diff21 =  ir & 0x1FFFFFllu; 
  if(diff21 & BIT20) diff21 |= EXTND21;
  return  diff21;
}
DATA_TYPE diff16(unsigned int ir){
  long long diff16;
  diff16 =  ir &   0xFFFFllu; 
  if(diff16 & BIT15) diff16 |= EXTND16;
  return  diff16;
}

instruction::instruction(){ /***** Constructor *****/
  memset(this, 0, sizeof(instruction));
}

void instruction::Fetch(ADDR_TYPE pc){
  static ULL tc=100; // you can select an initial value.
  Rav=Rbv=Rmv=Adr=Rcv=0;
  
  tag = tc++;
  ir  = Imem.instruction_load(pc);
  cpc = pc;
  iclass = get_iclass(ir);
  isort  = get_isort(ir);
  if(iclass==-1){
    cout << "PC 0x" << hex << cpc << "Error! Unimplemented.\n";
  }
}

void instruction::Decode(){
  switch(iclass){
  case EX_OPM: LW=31;     LR1=31;     LR2=31;     LR3=31;    break;
  case EX_PAL: LW=31;     LR1=31;     LR2=31;     LR3=31;    break;
  case EX_ADR: LW=ra(ir); LR1=31;     LR2=rb(ir); LR3=31;    break;
  case EX_JMP: LW=ra(ir); LR1=31;     LR2=rb(ir); LR3=31;    break;
  case EX_BRA: LW=ra(ir); LR1=31;     LR2=31;     LR3=31;    break;
  case EX_CBI: LW=31;     LR1=ra(ir); LR2=31;     LR3=31;    break;
  case EX_CBF: LW=31;     LR1=ra(ir); LR2=31;     LR3=31;    break;
  case EX_LDI: LW=ra(ir); LR1=31;     LR2=rb(ir); LR3=31;    break;
  case EX_LDF: LW=ra(ir); LR1=31;     LR2=rb(ir); LR3=31;    break;
  case EX_STI: LW=31;     LR1=ra(ir); LR2=rb(ir); LR3=31;    break;
  case EX_STF: LW=31;     LR1=ra(ir); LR2=rb(ir); LR3=31;    break;
  case EX_CMF: LW=rc(ir); LR1=ra(ir); LR2=rb(ir); LR3=rc(ir);break;
  case EX_FLO: LW=rc(ir); LR1=ra(ir); LR2=rb(ir); LR3=31;    break;
  case EX_CMI: LW=rc(ir); LR1=ra(ir); LR2=rb(ir); LR3=rc(ir);break;
  case EX_ARI: LW=rc(ir); LR1=ra(ir); LR2=rb(ir); LR3=31;    break;
  }

  switch(iclass){  /*** Decode Load/Store, memory access bytes ***/
  case EX_LDI:
  case EX_LDF:
  case EX_STI:
  case EX_STF:
    switch(code(ir)){
    case 0x28:  /** LDL **/
    case 0x22:  /** LDS **/
    case 0x2c:  /** STL **/
    case 0x26:  /** STS **/
      mem_byte = 4;
      break;
    case 0x0b:  /** LDQ_U **/
    case 0x23:  /** LDT **/
    case 0x29:  /** LDQ **/
    case 0x0a:  /** LDUB, gdb says so **/
    case 0x2d:  /** STQ **/
    case 0x0f:  /** STQ_U **/
    case 0x27:  /** STT */
    case 0x0e:  /** STB, gdb says so **/
    case 0x0d:  /** STW, gdb says so **/
      mem_byte = 8;
      break;
    default:
      cout << "Warning Decode ld/st default.\n";
    }
  }
  
}

void instruction::RegisterRead(){
  switch(iclass){
  case EX_OPM: Rav=0;          Rbv=0;         Rmv=0;        break;
  case EX_PAL: Rav=func26(ir); Rbv=0;         Rmv=0;        break;
  case EX_ADR: Rav=diff16(ir); Rbv=as.r[R2];  Rmv=0;        break;
  case EX_JMP: Rav=0;          Rbv=as.r[R2];  Rmv=0;        break;
  case EX_BRA: Rav=diff21(ir); Rbv=0;         Rmv=0;        break;
  case EX_CBI: Rav=as.r[R1];   Rbv=diff21(ir);Rmv=0;        break;
  case EX_LDI: Rav=0;          Rbv=as.r[R2];  Rmv=0;        break;
  case EX_LDF: Rav=0;          Rbv=as.r[R2];  Rmv=0;        break;
  case EX_STI: Rav=as.r[R1];   Rbv=as.r[R2];  Rmv=0;        break;
  case EX_STF: Rav=as.f[R1];   Rbv=as.r[R2];  Rmv=0;        break;
  case EX_CBF: Rav=as.f[R1];   Rbv=diff21(ir);Rmv=0;        break;
  case EX_FLO: Rav=as.f[R1];   Rbv=as.f[R2];  Rmv=0;        break;
  case EX_CMF: Rav=as.f[R1];   Rbv=as.f[R2];  Rmv=as.f[R3]; break;
  case EX_ARI: Rav=as.r[R1];                  Rmv=0;
    if(ir & BIT12) Rbv=lit8(ir); else Rbv=as.r[R2]; break;
  case EX_CMI: Rav=as.r[R1]; Rmv=as.r[R3];
    if(ir & BIT12) Rbv=lit8(ir); else Rbv=as.r[R2]; break;
  }
}

void instruction::Rename(){
  W  = LW;
  R1 = LR1;
  R2 = LR2;
  R3 = LR3;
}

void instruction::Execute(){
  ADDR_TYPE npc=cpc+4;
  switch(iclass){
  case EX_ADR: Rcv=calculate_adr(code(ir), Rbv, Rav); break;
  case EX_LDI:
  case EX_LDF:
  case EX_STI:
  case EX_STF: Adr=calculate_adr(code(ir), Rbv, diff16(ir)); break;
  case EX_CMI: 
  case EX_CMF: Rcv=ex_cmov(code(ir),func11(ir),Rav,Rbv,Rmv); break;
  case EX_ARI: 
  case EX_FLO: Rcv=ex_arith(code(ir),func11(ir),Rav,Rbv); break;
  case EX_JMP: Rcv=execute_jmp(&npc, Rbv); break;
  case EX_BRA: Rcv=execute_branch(&npc, Rav); break;
  case EX_CBI:
  case EX_CBF: ex_branch(code(ir), &npc, Rav, Rbv); break;
  }
  switch(iclass){
  case EX_JMP:
  case EX_BRA:
  case EX_CBI:
  case EX_CBF: as.pc=npc; t_pc=npc;
  }
}

void instruction::Memory(){
  switch(iclass){
  case EX_LDI:
  case EX_LDF:
    if(Adr%4!=0) cout << "Warning: Load 4byte alignment miss.\n";
    Rcv = Dmem.load(mem_byte, Adr);
    Rcv = get_load_data(code(ir), Rcv);
    break;
  }
}

void instruction::WriteBack(){
  switch(iclass){
  case EX_PAL: execute_pal(this); break;
  case EX_STI: 
  case EX_STF:
    if(Adr%4!=0) cout << "Warning: Store 4byte alignment miss.\n";
    Dmem.store(mem_byte, Adr, get_store_data(code(ir), Rav));
    break;
  case EX_JMP:
  case EX_BRA:
  case EX_ADR:
  case EX_LDI:
  case EX_CMI:
  case EX_ARI: as.r[W]=Rcv; break;
  case EX_LDF:
  case EX_CMF:
  case EX_FLO: as.f[W]=Rcv; break;
  }
}
