/* load_store.cc        This is a part of SimuAlpha Version 2.2a
 *                      Copyright (c) 1999,2000,2001, Kenji KISE
 ***************************************************************/
#include <stdio.h>
#include <iostream.h>
#include "define.h"

extern Dmemory Dmem;

DATA_TYPE long map_s(unsigned int int_tmp){
  unsigned int i8;
  
  i8 = (int_tmp >> 23) & 0xFF;
  
  if(i8==0xFF) return 0x00000000000007FFull;
  if(i8==0x00) return 0x0000000000000000ull;
  
  if(i8 & 0x80)    return 0x0000000000000400ull | (i8 & 0x7F);
  else             return 0x0000000000000380ull | (i8 & 0x7F);
  
  printf("Warning In load_store.c, map_s Error. \n");
}

DATA_TYPE get_store_data(int code, DATA_TYPE Rav){
  DATA_TYPE tmp;
  DATA_TYPE dat=0;
  int int_tmp;
  
  switch(code){
  case 0x2d: /** STQ   **/
  case 0x0f: /** STQ_U **/
  case 0x27: /** STT   **/
  case 0x0e: /** STB, gdb says so **/
  case 0x0d: /** STW, gdb says so **/
  case 0x2c: /** STL **/
    dat = Rav;
    break;
  case 0x26: /** STS **/
    tmp = Rav;
    int_tmp = 0;
    if(tmp & BIT63) int_tmp |= BIT8;       /* for S */
    int_tmp |= (tmp >> 52) & 0xFF;         /* for Exp. */
    int_tmp = int_tmp << 23;
    int_tmp |= ((tmp >> 29) & 0x7FFFFFlu); /* Frac. Hi and Lo */
    dat = (DATA_TYPE)int_tmp;
    break;
  default:
    cout << "Warning: default in get_store_data.\n";
    dat=0;
  }
  return dat;
}
  
DATA_TYPE get_load_data(int code, DATA_TYPE Rav_tmp){
  DATA_TYPE Rav;
  switch(code){
  case 0x28:  /** LDL **/
    Rav = Rav_tmp;
    if(Rav & BIT31) Rav|=EXTND32;
    break;
  case 0x22:  /** LDS **/
    Rav = 0llu | Rav_tmp >> 31;
    Rav = (Rav << 11)  | map_s(Rav_tmp);  /* MAP_S */
    Rav = (Rav << 23) | (Rav_tmp & 0x7FFFFFlu);
    Rav = Rav << 29;
    break;
  case 0x29:  /** LDQ **/
  case 0x0b:  /** LDQ_U **/
  case 0x23:  /** LDT **/
  case 0x0a:  /** LDUB, gdb says so **/
    Rav = Rav_tmp;
    break;
  default:
    cout << "Warning: default in get_load_data.\n";
    Rav=0;
  }
  return Rav;
}

/************************************************************/
ADDR_TYPE calculate_adr(int code, DATA_TYPE Rbv, DATA_TYPE dif){
  ADDR_TYPE adr=0;
  switch(code){
  case 0x08:  adr = Rbv + dif; break;
  case 0x09:  adr = Rbv + (dif << 16); break;
  case 0x28:  adr = Rbv + dif; break;
  case 0x29:  adr = Rbv + dif; break;
  case 0x0b:  adr = (Rbv + dif) & 0xFFFFFFFFFFFFFFF8llu; break;
  case 0x2d:  adr = Rbv + dif; break;
  case 0x2c:  adr = Rbv + dif; break;
  case 0x0f:  adr = (Rbv + dif) & 0xFFFFFFFFFFFFFFF8llu; break;
  case 0x27:  adr = Rbv + dif; break;
  case 0x23:  adr = Rbv + dif; break;
  case 0x22:  adr = Rbv + dif; break;
  case 0x26:  adr = Rbv + dif; break;
  case 0x0a:  adr = Rbv + dif; break;
  case 0x0e:  adr = Rbv + dif; break;
  case 0x0d:  adr = Rbv + dif; break; /* STW */
  }
  return adr;
}
