/* mem_init.cc          This is a part of SimuAlpha Version 2.2a
 *                      Copyright (c) 1999,2000,2001, Kenji KISE
 ***************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "define.h"

extern class architecture_state as;
extern struct machine_setting ms;
extern Imemory Imem;
extern Dmemory Dmem;

ADDR_TYPE mem_init_instruction(char *program_name){
  ADDR_TYPE ibuffer_min_adr;  /* minimun address of Ibuffer */
  ADDR_TYPE ibuffer_max_adr;  /* maximun address of Ibuffer */
  FILE *fp;
  FILE *fp_tmp;
  int i,j;
  struct FF_filehdr hdr;
  struct FF_aouthdr aout;
  struct FF_scnhdr  st;
  unsigned int *data;

  if((fp = fopen(program_name, "r")) == NULL) {
    fprintf(stderr, "Bad file name: %s\n", program_name);
    exit(0);
  }
  fp_tmp = fopen(program_name, "r");
  
  /* read header */
  fread(&hdr, sizeof(hdr), 1, fp);
  fread(&aout,sizeof(aout),1, fp);
  if(hdr.f_magic!=387){
    printf("%s not executable.\n", program_name); exit(0);}
  
  for(i=0; i<hdr.f_nscns; i++){
    fread(&st,sizeof(st),1, fp);
    
    if(!strncmp(st.s_name, ".dynamic",8)){
      printf("This program may use shared library.\n");
      printf("Compile with -static for gcc or -non_shared for Digital cc.\n");
      exit(1);
    }
    
    if(!strcmp(st.s_name, ".text")  || !strcmp(st.s_name, ".init")){
      if(!strcmp(st.s_name, ".text")){
	st.s_size = aout.tsize;
	ibuffer_min_adr = st.s_paddr;
	ibuffer_max_adr = st.s_paddr + st.s_size;
	Imem.Imemory_init(st.s_size, 
			  ibuffer_min_adr, ibuffer_max_adr);
      }
      
      data = (unsigned int *)malloc(st.s_size); 
      fseek(fp_tmp, st.s_scnptr, SEEK_SET);
      fread(data, st.s_size, 1, fp_tmp);
      for(j=0; j<(st.s_size)/4; j++){
	Imem.instruction_store(st.s_paddr + 4*j, data[j]);
      }
      free(data);
    }
  }
  fclose(fp_tmp);
  fclose(fp);
  return aout.entry;
}


void mem_init(ARCH_STATE *as, int argc, char **argv, char **argpt){
  ADDR_TYPE sbuffer_min; /* Minimun Address of Stack Buffer */
  ADDR_TYPE sbuffer_max; /* Maximun Address of Stack Buffer */
  ADDR_TYPE dbuffer_min; /* Minimun Address of Data Buffer  */
  ADDR_TYPE dbuffer_max; /* Maximun Address of Data Buffer  */
  FILE *fp;
  FILE *fp_tmp;
  int i,j;
  int argp_count;
  char **argp;
	       
  struct FF_filehdr hdr;
  struct FF_aouthdr aout;
  struct FF_scnhdr st;
  ADDR_TYPE ptr, str_ptr;
  unsigned int *data;
  char *program_name;
  char *char_ptr;

  char *argp_sample[] ={
    "TERM=xterm",
    "USER=kis",
    "HOSTTYPE=alpha",
    NULL};
  
  program_name = argv[0];
  as->pc = mem_init_instruction(program_name);/* Set Instr. */
  
  if(ms.current_argp_flag==0) argp = argp_sample;
  else                        argp = argpt;  
  
  if((fp = fopen(program_name, "r")) == NULL) {
    fprintf(stderr, "Bad file name: %s\n", program_name);
    exit(1);
  }
  fp_tmp = fopen(program_name, "r");
  
  /* read header */
  fread(&hdr, sizeof(hdr), 1, fp);
  fread(&aout,sizeof(aout),1, fp);

  if(hdr.f_magic!=387){
    printf("%s is not an executable file.\n", program_name);
    exit(0);
  }

  dbuffer_min=0x140000000llu;
  dbuffer_max= aout.bsize + aout.bbs_start; /* 0x150000000llu; */
  Dmem.set_dbuffer(dbuffer_min, dbuffer_max);
  sbuffer_min=0x110000000llu;
  sbuffer_max=0x110000000llu;
  Dmem.set_sbuffer(sbuffer_min, sbuffer_max);
  
  for(i=0; i<hdr.f_nscns; i++){
    fread(&st,sizeof(st),1, fp);
    
    if(!strcmp(st.s_name, ".data") ||
       !strcmp(st.s_name, ".rdata") || 
       !strcmp(st.s_name, ".rconst")){
      
      if(!strcmp(st.s_name, ".data")){
	st.s_size = aout.dsize;
      }
      if(!strcmp(st.s_name,".rdata") || 
	 !strcmp(st.s_name,".rconst")){
	if((ADDR_TYPE)(st.s_paddr+st.s_size)>sbuffer_max){
	  sbuffer_max = st.s_paddr+st.s_size;
	  Dmem.set_sbuffer(sbuffer_min, sbuffer_max);
	}
      }
      if((st.s_size%4)!=0) 
	printf("st.s_size is not devided by 4.\n");

      data = (unsigned int *)malloc(st.s_size); 
      fseek(fp_tmp, st.s_scnptr, SEEK_SET);
      fread(data, st.s_size, 1, fp_tmp);
      for(j=0; j<(st.s_size)/4; j++){
	Dmem.store_4byte(st.s_paddr + j*4, data[j]);
      }
      free(data);
    }
  }
  
  /*** memory setting, SP, ARGV, ARGC etc. ***/
  for(argp_count=0; 1; argp_count++)
    if(argp[argp_count]==NULL) break;

  /* Decide the value of SP, this value has no special meaning.
     you can decide appropriate address. */
  if(ms.current_argp_flag==0) ptr = as->r[30] = 0x11ffff000ull;
  else ptr = as->r[30] = ms.current_argp_flag;
  
  str_ptr = as->r[30] + (1 + argc + 1 + argp_count + 5)*8;
  Dmem.store_8byte(ptr, argc);   /* write argc to the memory */
  ptr+=8;
  
  for(i=0; i<argc; i++){         /* write argv to the memory */
    Dmem.store_8byte(ptr, str_ptr);
    ptr+=8;
    
    char_ptr = argv[i];
     for(j=0; j<(int)strlen(argv[i]); j++){
      Dmem.store_byte(str_ptr, *(char_ptr + j));
      str_ptr++;
    }
    Dmem.store_byte(str_ptr, (char)NULL);
    str_ptr++;
  }
  ptr+=8;  /* for NULL after ARGV */
  
  for(i=0; i<argp_count; i++){      /* write argp to the memory */
    Dmem.store_8byte(ptr, str_ptr);
    ptr+=8;
    char_ptr = argp[i];
    for(j=0; j<(int)strlen(argp[i]); j++){
      Dmem.store_byte(str_ptr, *(char_ptr + j));
      str_ptr++;
    }
    Dmem.store_byte(str_ptr, *(char_ptr + j));
    str_ptr++;
  }
  return;
}
