/* memory.cc            This is a part of SimuAlpha Version 2.2a
 *                      Copyright (c) 1999,2000,2001, Kenji KISE
 ***************************************************************/
#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include "define.h"

extern struct evaluate e;

void Imemory::Imemory_init(int size, ADDR_TYPE min, ADDR_TYPE max){
  ibuffer = (unsigned int *)malloc(size);
  ibuffer_min_adr=min;
  ibuffer_max_adr=max;
}
unsigned int Imemory::instruction_load(ADDR_TYPE adr){
  if(adr<ibuffer_min_adr || adr>ibuffer_max_adr){
    return 0;
  }
  return ibuffer[(adr - ibuffer_min_adr)/4];
}
void Imemory::instruction_store(ADDR_TYPE adr, unsigned int data){
  if(adr<ibuffer_min_adr || adr>ibuffer_max_adr){
    printf("Error: Instruction_store as->pc is out of range.\n");
    exit(1);
  }
  ibuffer[(adr-ibuffer_min_adr)/4] = data;
}
/**************************************************/
void Dmemory::set_dbuffer(ADDR_TYPE min, ADDR_TYPE max){
  dbuffer_min = min;
  dbuffer_max = max;
}

/* Systemcall obreak use the function */
void Dmemory::set_dbuffer(ADDR_TYPE max){
  dbuffer_max = max;
}

void Dmemory::set_sbuffer(ADDR_TYPE min, ADDR_TYPE max){
  sbuffer_min = min;
  sbuffer_max = max;
}

/*
 * Translation from a logical address to a  phisical address.
 **********************************************************/
char* Dmemory::adr_translate(ADDR_TYPE adr){
  ADDR_TYPE index;
  ADDR_TYPE offset;
  int i;

  if(adr<sbuffer_min || adr>dbuffer_max ||
     (adr>sbuffer_max && adr<dbuffer_min)){
    //    cout << "Warning: invalid address: " << hex <<  adr << "\n";
    return NULL;
  }
    
  index  = adr >> PAGE_BIT;
  offset = adr & PAGE_MASK;  /* page offset */
  
  for(i=0; i<TLB_ENTRY; i++){
    if(tlb[i].valid==0){     /* TLB miss */
      e.used_memory++;
      tlb[i].valid = 1;
      tlb[i].adr   = index;
      tlb[i].ptr   = (char *)calloc(1, MEM_PAGE_SIZE);
      if(tlb[i].ptr==NULL){
	cout << "Error: in data memory allocation. "
	     << dec << i << "MB used.\n";
	exit(1);
      }
      return tlb[i].ptr;
    }  
    if(tlb[i].adr == index){ /* tlb hit */
      return tlb[i].ptr;
    }
  }
  printf("TLB is full. Enlarge TLB_ENTRY.\n");  exit(1);
}


/************************************************************/
char Dmemory::load_byte(ADDR_TYPE adr){
  unsigned int offset;
  char *ptr;
  
  offset = adr & PAGE_MASK;  /* page offset */
  ptr = adr_translate(adr);
  if(ptr==NULL) return 0;
  return ptr[offset];
}

unsigned int Dmemory::load_4byte(ADDR_TYPE adr){
  unsigned int offset;
  char *ptr;
  int dat;
  unsigned int ret;

  offset = adr & PAGE_MASK;  /* page offset */

  if((offset+3)>=MEM_PAGE_SIZE) {
    //    printf("Warning: In memory.c, load_4byte, offset+3>=PAGE.\n");
    ret=0;
    dat = load_byte(adr+3); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+2); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+1); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+0); ret |= (dat & 0xFF);
    return ret;
  }
  ptr = adr_translate(adr);
  if(ptr==NULL) return 0;
  memcpy(&ret, &ptr[offset], 4);
  return  ret;
}

DATA_TYPE Dmemory::load_8byte(ADDR_TYPE adr){
  ADDR_TYPE offset;
  DATA_TYPE ret;
  char *ptr;
  int dat;
  
  offset = adr & PAGE_MASK;  /* page offset */
  if((offset+7)>=MEM_PAGE_SIZE){
    //    printf("Warning: adr 0x%qx In memory.c, load_8byte, offset+7>=PAGE.\n",adr);
    ret=0;
    dat = load_byte(adr+7); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+6); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+5); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+4); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+3); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+2); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+1); ret |= (dat & 0xFF); ret = ret << 8;
    dat = load_byte(adr+0); ret |= (dat & 0xFF);
    return ret;
  }
  ptr = adr_translate(adr);
  if(ptr==NULL) return 0;
  memcpy(&ret, &ptr[offset], 8);
  return ret;
}

DATA_TYPE Dmemory::load(int byte, ADDR_TYPE adr){
  if(byte==4) return load_4byte(adr);
  if(byte==8) return load_8byte(adr);
  cout << "Warning: Dmemory::load, byte is not 4 and 8.\n";
  return 0;
}

/************************************************************/
void Dmemory::store_byte(ADDR_TYPE adr, char data){
  unsigned int offset;
  char *ptr;
  
  offset = adr & PAGE_MASK;  /* page offset */
  ptr = adr_translate(adr);
  if(ptr==NULL){ cout << "Store Adr Error.\n"; exit(1);}
  ptr[offset]=data;
}

void Dmemory::store_4byte(ADDR_TYPE adr, unsigned int data){
  unsigned int offset;
  char *ptr;

  offset = adr & PAGE_MASK;  /* page offset */
  if((offset+3)>=MEM_PAGE_SIZE) {
    printf("In memory.c, store_4byte Unimplemented Error.\n");
  }
  ptr = adr_translate(adr);
  if(ptr==NULL){ cout << "Store Adr Error.\n"; exit(1);}
  memcpy(&ptr[offset], &data, 4);
}

void Dmemory::store_8byte(ADDR_TYPE adr, DATA_TYPE data){
  ADDR_TYPE offset;
  char *ptr;
  
  offset = adr & PAGE_MASK;  /* page offset */
  if((offset+7)>=MEM_PAGE_SIZE) {
    printf("In memory.c, store_8byte Unimplemented Error.\n");
  }
  ptr = adr_translate(adr);
  if(ptr==NULL){ cout << "Store Adr Error.\n"; exit(1);}
  memcpy(&ptr[offset], &data, 8);
}

void Dmemory::store(int byte, ADDR_TYPE adr, DATA_TYPE data){
  if(byte==4){
    store_4byte(adr, (unsigned int)(data & MASK32)); return; }
  if(byte==8){ 
    store_8byte(adr, data); return; }
  cout << "Warning: Dmemory::store, byte is not 4 and 8.\n";
}
