/* simu.cc              This is a part of SimuAlpha Version 2.2a
 *                      Copyright (c) 1999,2000,2001, Kenji KISE
 ***************************************************************/
#include <iomanip.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "define.h"

static char UsageMessage[] =
  "SimuAlpha version 2.2a       last updated 2001.01.25\n\
Usage: SimuAlpha [-option] command\n\
 -d: debug mode(set ms.debug_flag).\n\
 -s: print trace of systemcall(set ms.syscall_flag).\n\
 -e[num][m]: specify end_tc. m stands for million. \n\
 -p[SP]: use argp of the current system. Debugging option.\n\
 -V[num]:  verbose mode. prints '.' by VERBOSE_INTERVAL .\n\
";

class architecture_state as;
struct machine_setting ms;
struct evaluate e;

Imemory Imem; // Instruction Memory
Dmemory Dmem; // Data Buffer

void print_result(ostream& out){
  out << "Program exited normally. ";
  out << dec << e.retired_inst;
  out << " code executed. " << e.used_memory <<  "MB used.\n";
}

void construct_data(int argc, char **argv, char **argp){
  int i;
  char *buf;
  buf = (char *)malloc(1024);

  memset(&as, 0, sizeof(as));
  memset(&ms, 0, sizeof(ms));
  memset(&e,  0, sizeof(e) );
  if(argc>1){
    for(i=0; i<10; i++){
      if(argc==1) break;
      if((*(char *)(argv[1]))=='-'){
	switch( *((char *)(argv[1]) +1) )
	  {
	  case 'h': cout << UsageMessage; exit(0);
	  case 'd': ms.debug_flag   = 1;           break;
	  case 's': ms.syscall_flag = 1;           break;
	  case 't': ms.distrib_flag = 1;           break;
	  case 'V':
	    ms.verbose_interval =atoi((char *)(argv[1])+2);
	    if(ms.verbose_interval==0) ms.verbose_interval=100000;
	    if(*((char *)(argv[1])+strlen(argv[1])-1)=='m')
	      ms.verbose_interval *=1000000;
	    cout << "verbose interval:"
		 <<dec <<  ms.verbose_interval << "\n";
	    break;
	  case 'p':
	    ms.current_argp_flag= get_hex((char *)(argv[1])+2);
	    if(ms.current_argp_flag==0) 
	      ms.current_argp_flag = 0x11ffff000ull;
	    cout << "Use sp:" << hex 
		 << ms.current_argp_flag << "\n";
	    break;
	  case 'e':
	    ms.end_tc =atol((char *)(argv[1])+2);
	    if(*((char *)(argv[1])+strlen(argv[1])-1)=='m')
	      ms.end_tc *=1000000;
	    printf("End tc: %.0f\n", (double)ms.end_tc); 
	    break;
	  default:
	    fprintf(stderr, "Error: Wrong option(s)!!\n\n");
	    cout << UsageMessage;
	    exit(0);
	  }
	argv++;
	argc--;
      }
    }
  }
  if(argc < 2){cout << UsageMessage; exit(0);}
  argv++;
  argc--;

  mem_init(&as, argc, argv, argp);
}

void destruct_simulator(){
  print_result(cout);
  exit(0);
}

void main_loop(){
  instruction *p = new instruction;
  while(1){
    as.r[31]=0;   
    as.f[31]=0;
    
    if(ms.debug_flag) debugmode();

    p->Fetch(as.pc);   as.pc+=4;
    p->Decode();
    p->Rename();
    p->RegisterRead();
    p->Execute();
    p->Memory();
    p->WriteBack();
    if(p->iclass==EX_PAL) as.pc = p->t_pc;
    e.retired_inst++;

    if(ms.verbose_interval && 
       (e.retired_inst%ms.verbose_interval)==0){
      fprintf(stdout, "."); fflush(stdout);
    }
    if(ms.end_tc!=0 && e.retired_inst>=ms.end_tc)
      destruct_simulator();
    if(ms.end_flag==1) destruct_simulator();
  }
}

int main(int argc, char **argv, char **argp){
  construct_data(argc, argv, argp);
  main_loop();
  return 0;
}
