/* syscall.cc           This is a part of SimuAlpha Version 2.2a
 *                      Copyright (c) 1999,2000,2001, Kenji KISE
 ***************************************************************/
#define	SYS_sigreturn	 103
#define	SYS_sigprocmask	  48
#define	SYS_obreak	  17
#define	SYS_getrlimit	 144
#define	SYS_setrlimit	 145
#define	SYS_stat	  67
#define	SYS_fstat	  91
#define	SYS_ioctl	  54
#define	SYS_sigaction	 156
#define	SYS_close	   6
#define	SYS_read	   3
#define	SYS_write	   4
#define	SYS_open	  45
#define	SYS_unlink	  10
#define	SYS_getpagesize	  64
#define	SYS_gettimeofday 116
#define	SYS_getrusage	 117
#define	SYS_lseek	  19
#define	SYS_getuid	  24

#include "memory.h"
#include "define.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>

#if defined(__i386)
#include <unistd.h> 
#endif

#include <iostream.h>
#include <iomanip.h>
#include <stdio.h>
#include <string.h>
#include <sys/resource.h>
#include "code.h"

extern Dmemory Dmem;

extern class architecture_state as;
extern struct machine_setting ms;
extern struct evaluate e;

extern void destruct_simulator();

/************************************************************/
void syscall(instruction *p){
  char buf[8192*10];
  char buf2[8192];
  char name[20];
  long long ret=0;
  long long i;

  p->t_pc = p->cpc+4;
  
  p->cache_flush = 0;
  switch(as.r[0]){
    
  case SYS_sigreturn:
    {
      int i;
      unsigned long long buf[35+1+48];
      
      ret = as.r[17];
      for(i=0; i<84; i++) buf[i] = Dmem.load_8byte(as.r[16]+i*8);
      for(i=0; i<31; i++) as.r[i] = buf[i+4];
      /*  for(i=0; i<31; i++){as.f[i] = buf[i+39]; } */
      
      p->t_pc = buf[2];
      
      //      as.sigreturn_pc = buf[2];
      strcpy(name, "sigreturn");
      p->cache_flush = 1;
      if(ms.debug_flag){
	cout << "Sigreturn: return Adr: " << hex << p->t_pc << "\n";
      }
      break;
    }
    
  case SYS_sigprocmask:
    ret = 0;
    strcpy(name, "sigprocmask");
    if(as.r[17] == 0xfffffffffffff137ull){/* I don't know why. GDB say so. */
      ms.end_flag = 1;
    }
    break;

  case SYS_obreak:
    ret = 0;
    { /* I am not sure, true? */
      unsigned long long max = as.r[16] + as.r[18];
      Dmem.set_dbuffer(max);
    }
    strcpy(name, "obreak");
    break; 

  case SYS_getrlimit:
    for(i=0; i<sizeof(struct rlimit); i++){
      buf2[i] = Dmem.load_byte(as.r[17]+i);
    }
    
    ret = getrlimit((enum __rlimit_resource)as.r[16], (struct rlimit *)buf2);
    // ret = getrlimit(as.r[16], (struct rlimit *)buf2);
    strcpy(name, "getrlimit");
    p->cache_flush = 1;
    break; 

  case SYS_setrlimit:
    /* Do nothing. */
    /* for(i=0; i<sizeof(struct rlimit); i++){
       buf2[i] = Dmem.load_byte(as.r[17]+i);    }
       ret = setrlimit(as.r[16], buf2); */
    ret = 0;
    strcpy(name, "setrlimit");
    break; 

  case SYS_stat:
    printf("stat executed.\n");
    for(i=0; i<1024; i++){
      buf2[i] = Dmem.load_byte(as.r[16]+i);
    }
    for(i=0; i<sizeof(struct stat); i++){
      buf[i] = Dmem.load_byte(as.r[17]+i);
    }
    ret = stat(buf2, (struct stat *)buf);
    strcpy(name, "stat");
    p->cache_flush = 1;
    break; 

  case SYS_fstat:
    {
      char buf[256];
      ret = fstat(as.r[16], (struct stat *)buf);

      /**** fstat set 0, bcause FreeBSD and Degital Unix's
	implementations are different. >> Store 0 ****/
      
      for(i=0; i<10; i++) Dmem.store_8byte(as.r[17]+i*8, 0); 
      
      /* for(i=0; i<sizeof(struct stat); i++){
	Dmem.store_byte(as.r[17]+i, buf[i]);
	printf("%x ", buf[i]); 
      } */
      
      strcpy(name, "fstat");
      p->cache_flush = 1;
      break; 
    }
    
  case SYS_ioctl:
    /*    ret = ioctl(as.r[16], as.r[17],  */
    strcpy(name, "ioctl");
    ret = 0;
    break;

  case SYS_sigaction:
    ret = 0;
    /*    ret = sigaction(as.r[16], as.r[17], as.r[18]); */
    strcpy(name, "sigaction");
    break;

  case SYS_close:
    if(as.r[16]>2) ret = close(as.r[16]); else ret=0;
    strcpy(name, "close");
    break;

  case SYS_read:
    ret = read((int)as.r[16], (void *)buf, (size_t)as.r[18]);
    for(i=0; i<(long long)ret; i++){
      Dmem.store_byte(as.r[17]+i, buf[i]);
    }
    p->cache_flush = 1;
    strcpy(name, "read");
    break;

  case SYS_lseek:
    ret = lseek(as.r[16], as.r[17], as.r[18]);
    strcpy(name, "lseek");
    break;
		
  case SYS_write:
    for(i=0; i<(long long)as.r[18]; i++){
      buf[i] = Dmem.load_byte(as.r[17]+i);
    }
    ret = write((long)as.r[16], (void *)buf, (long)as.r[18]);
    p->cache_flush = 1;
    strcpy(name, "write");
    if(ret==-1){ printf("Syscall Write Error!\n"); exit(1);}
    break;
    
  case SYS_open:
    for(i=0; i<1024; i++){
      buf[i] = Dmem.load_byte(as.r[16]+i);
    }
    ret = open(buf, as.r[17], as.r[18]);
    strcpy(name, "open");
    break;

  case SYS_unlink:
    for(i=0; i<1024; i++){
      buf[i] = Dmem.load_byte(as.r[16]+i);
    }
    ret = unlink(buf);
    p->cache_flush = 1;
    strcpy(name, "unlink");
    break;

    
  case SYS_getuid:
    ret = getuid();
    strcpy(name, "getuid");
    break;
    
  case SYS_getpagesize:
    ret = 8192; /* getpagesize(); */
    strcpy(name, "getpagesize");
    break;

  case SYS_gettimeofday:
    {
      struct timeval tv;
      struct timezone tz;
      
      ret = gettimeofday(&tv, &tz);
      memcpy(buf, &tv, sizeof(tv));
      for(i=0; i<sizeof(tv); i++){
	Dmem.store_byte(as.r[16]+i, buf[i]);
      }
      p->cache_flush = 1;          
      strcpy(name, "gettimeofday");
      break; 
    }
    
  case SYS_getrusage:
    {
      struct rusage ru; 
      ret = getrusage((enum __rusage_who)as.r[16], &ru);
      //     ret = getrusage(as.r[16], &ru);
      memcpy(buf, &ru, sizeof(ru));
      for(i=0; i<sizeof(ru); i++){
	Dmem.store_byte(as.r[17]+i, buf[i]);
      }
      p->cache_flush = 1;
      strcpy(name, "getrusage");
      break;
    }
    
  default:
    cout << "TC:" << e.retired_inst;
    cout << "  Syscall.c Default Error!!\n";
    cout << "Syscall No:" << as.r[0];
    cout << "   Not implemented.\n";
    destruct_simulator();
  }

  if(ms.syscall_flag){
    cout << "TC " << setw(9) << dec << e.retired_inst;
    cout << ":     " << name;
    cout << "  "    << hex << as.r[16];
    cout << ", "    << hex << as.r[17];
    cout << ", "    << hex << as.r[18];
    cout << " >> "  << hex << ret << "\n";
  }
  fflush(stdout);
  
  as.r[1]=as.r[2]=as.r[3]=as.r[4]=as.r[5]=as.r[6]=as.r[7]=as.r[8]=0;
  as.r[19]=as.r[20]=as.r[22]=as.r[24]= as.r[25]=as.r[28]=0;
  as.r[0] = ret;
  as.r[23]= p->t_pc;
}


/************************************************************/
void execute_pal(instruction *p){
  p->t_pc = p->cpc+4;
  switch(func26(p->ir) & 0xffff)
    {
    case 0x0000: 
      cout << "TC:" << dec << e.retired_inst << "pal_halt\n"; exit(1);
    case 0x0086: cout << "imb\n";  exit(1);
    case 0x0083: syscall(p); break;
    default:
      if(ms.syscall_flag)
	cout << "TC " << setw(9) << dec << e.retired_inst
	     << ":  Warning: in execute_pal default.\n";
    }
}
